/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors;

import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.ILogDAO;
import it.eng.spago4q.extractors.bo.ExtractorParameter;
import it.eng.spago4q.extractors.bo.GenericItemInterface;
import it.eng.spago4q.extractors.parameterfiller.DefaultExtractionParameterFiller;
import it.eng.spago4q.extractors.parameterfiller.ExtractionParameterFiller;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

public abstract class AbstractExtractor implements ExtractorInterface {

    private EOperation eOperation = null;
    private List<ExtractorParameter> operationParameter = null;
    private List<ExtractorParameter> dataSourceParameter = null;
    private static final String ELOG_DAO = "ELogDAO";
    private static transient Logger logger = Logger
            .getLogger(AbstractExtractor.class);

    public List<GenericItemInterface> execute() throws EMFUserError {
        List<GenericItemInterface> toReturn = null;
        setUp();
        try {
            toReturn = extract();
        } catch (Exception e) {
            logger.error(e.getLocalizedMessage());
            throw new EMFUserError(EMFErrorSeverity.ERROR, e
                    .getLocalizedMessage());
        } finally {
            tearDown();
        }
        return toReturn;
    }

    public void init(final List<ExtractorParameter> iDataSourceParameter,
            final List<ExtractorParameter> iOperationParameter) {
        if (iDataSourceParameter != null) {
            dataSourceParameter = new ArrayList<ExtractorParameter>();
            dataSourceParameter.addAll(iDataSourceParameter);
        }

        if (iOperationParameter != null) {
            operationParameter = new ArrayList<ExtractorParameter>();
            operationParameter.addAll(iOperationParameter);
        }
    }

    @Deprecated
    public void init(final EOperation operation) throws EMFUserError {
        this.eOperation = operation;
        ExtractionParameterFiller extractionParameterFiller = new DefaultExtractionParameterFiller();
        extractionParameterFiller.setOperation(operation);
        setOperationParameter(extractionParameterFiller
                .getOperationParameters());
        setDataSourceParameter(extractionParameterFiller.getDataSourceParameters());

    }

    protected EOperation getEOperation() {
        return eOperation;
    }

    public List<ExtractorParameter> getOperationParameter() {
        return operationParameter;
    }

    protected void setOperationParameter(
            final List<ExtractorParameter> iOperationParameter) {
        operationParameter = iOperationParameter;
    }

    protected void setDataSourceParameter(
            final List<ExtractorParameter> iDataSourceParameter) {
        dataSourceParameter = iDataSourceParameter;
    }

     public void setOperationParameter(final Integer operationId)
     throws EMFUserError {
         
     }

    public List<ExtractorParameter> getDataSourceParameter() {
        return dataSourceParameter;
    }

     public void setDataSourceParameter(final Integer dataSourceId)
     throws EMFUserError {
     }

    protected String readDataSourceParameterValue(final String key) {
        String toReturn = null;
        Iterator<ExtractorParameter> it = (Iterator<ExtractorParameter>) getDataSourceParameter()
                .iterator();
        while (it.hasNext()) {
            ExtractorParameter currentDataSourceParameter = (ExtractorParameter) it
                    .next();
            if (currentDataSourceParameter.getKey().equals(key)) {
                toReturn = currentDataSourceParameter.getValue();
            }
        }
        return toReturn;
    }

    protected List<String> readDataSourceParameterValueList(final String key) {
        List<String> toReturn = new ArrayList<String>();
        Iterator<ExtractorParameter> it = (Iterator<ExtractorParameter>) getDataSourceParameter()
                .iterator();
        while (it.hasNext()) {
            ExtractorParameter currentDataSourceParameter = (ExtractorParameter) it
                    .next();
            if (currentDataSourceParameter.getKey().equals(key)) {
                toReturn.add(currentDataSourceParameter.getValue());
            }
        }
        return toReturn;
    }

    protected String readOperationParameterValue(final String key) {
        String toReturn = null;
        Iterator<ExtractorParameter> it = (Iterator<ExtractorParameter>) getOperationParameter()
                .iterator();
        while (it.hasNext()) {
            ExtractorParameter currentOperationParameter = (ExtractorParameter) it
                    .next();
            if (currentOperationParameter.getKey().equals(key)) {
                toReturn = currentOperationParameter.getValue();
            }
        }
        return toReturn;
    }

    protected List<String> readOperationParameterValueList(final String key) {
        List<String> toReturn = new ArrayList<String>();
        Iterator<ExtractorParameter> it = (Iterator<ExtractorParameter>) getOperationParameter()
                .iterator();
        while (it.hasNext()) {
            ExtractorParameter currentOperationParameter = (ExtractorParameter) it
                    .next();
            if (currentOperationParameter.getKey().equals(key)) {
                toReturn.add(currentOperationParameter.getValue());
            }
        }
        return toReturn;
    }

    protected abstract void setUp() throws EMFUserError;

    protected abstract void tearDown() throws EMFUserError;

    protected abstract List<GenericItemInterface> extract() throws EMFUserError;

    public String dataSourceTest(final Integer dataSourceId)
            throws EMFUserError {
        return "No data source test found!";
    }

    public String operationTest(final Integer operationId) throws EMFUserError {
        return "No operation test found!";
    }

    public Date getLastExtractionDate() throws EMFUserError {
        Date toReturn = null;
        if (eOperation != null) {
            ILogDAO logDAO = (ILogDAO) DAOFactory.getDAO(ELOG_DAO);
            toReturn = logDAO.getLastOperationExecutionDate(eOperation.getId());
        }
        return toReturn;
    }
}
