/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors;

import it.eng.spago.error.EMFUserError;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.extractors.bo.GenericItemInterface;
import it.eng.spago4q.extractors.bo.InterfaceItem;
import it.eng.spago4q.extractors.script.ScriptExecutor;
import it.eng.spago4q.extractors.utilities.OperationUtil;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

public class TransformAndInsertUtil {
    
    private String archiveDir;

    public void init(final String archDir) {
        archiveDir = archDir;
    }

    protected void transformAndInsertData(final Integer processId,
            final EOperation operation, final Date operationDate,
            final Date executionDate, final ScriptExecutor scriptExecutor,
            final IEngUserProfile userProfile, String archivePath,
            final List<GenericItemInterface> genericItemList)
            throws EMFUserError {
        DomainValueCacheCreator domainValueCacheCreator = new DomainValueCacheCreator(
                operation.getInterfaceType().getId());
        if (operation.getArchive()) {
            archivePath = OperationUtil.createBeckUpArchive(genericItemList,
                    operationDate, operation, archiveDir);
        }

        LogWriter logWriter = new LogWriter();

        logWriter.writeExtractedItems(processId, operation.getId(),
                genericItemList.size(), operationDate, executionDate,
                archivePath);

        MapMaker mapMaker = new MapMaker();
        List<InterfaceItem> interfaceItem = mapMaker.mapping(operation,
                genericItemList, scriptExecutor, domainValueCacheCreator);

        String rejectedPath = null;
        if (mapMaker.getMappingErrorManager().getNumberOfRejected() > 0
                && operation.getRejected()) {
            rejectedPath = mapMaker
                    .getMappingErrorManager()
                    .createRejectedArchive(executionDate, operation, archiveDir);
        }
        logWriter.writeMappingError(mapMaker.getMappingErrorManager()
                .getNumberOfRejected(), rejectedPath);

        FactTableManager factTableManager = new FactTableManager(operation
                .getInterfaceType().getId());
        int numberOfInsert = 0;

        for (Iterator iterator2 = interfaceItem.iterator(); iterator2.hasNext();) {
            InterfaceItem interfaceItem2 = (InterfaceItem) iterator2.next();
            if (factTableManager.loadData(operationDate, operation
                    .getInterfaceType().getId(), interfaceItem2, logWriter
                    .getLogId(), userProfile)) {
                numberOfInsert++;
            }
        }
        logWriter.wirteInsertedItems(numberOfInsert);
    }

}
